// COPYRIGHT 


/*****************************************
 INCLUDES
*******************************************/


#include "kppInterfaceWalk/kppCommandLoadHRP2.h"

#include "KineoController/kppSetPropertyCommand.h"
#include "KineoModel/kppComponentParameter.h"
#include "KineoModel/kppValue.h"
#include "KineoModel/kppComponentClassFilter.h"
#include "KineoModel/kppComponent.h"
#include "KineoModel/kppSolidComponent.h"

#include "hppCorbaServer/hppciOpenHrp.h"

#include "KineoKCDModel/kppKCDPolyhedron.h"

#include "kppInterfaceWalk/openHrpConfig.h"
#include "hppWalkFootPlanner/hppWalkFootPlanner.h" 
#include "hppWalkFootPlanner/hppWalkSteeringMethod.h"
#include "hppWalkFootPlanner/hppWalkDistance.h"


#include "hrp2_14/hrp2_14.h"

/*****************************************
 DEFINES
*******************************************/
#define TOR_OBSTACLE 0.015
#define PENETRATION_BOX 0.01
#define OPT_MAX_NB  3

// Select level of verbosity at configuration.
#if DEBUG==2
#define ODEBUG2(x) std::cout << "walkMainCodels:" << x << std::endl
#define ODEBUG1(x) std::cerr << "walkMainCodels:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "walkMainCodels:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif

/*****************************************
 METHODS
*******************************************/

/* --- setDefaultRoadmapBuilderAndOptimizer -------------------------- */

static ktStatus setDefaultRoadmapBuilderAndOptimizer(ChppPlanner *inHppPlanner)
{


  // Set default tools for the robot bouding box.
  const ChppDeviceShPtr robotBox = 
    KIT_DYNAMIC_PTR_CAST(ChppDevice, inHppPlanner->robotIthProblem(0));

  if (!robotBox) {
    ODEBUG1(":walkInitMain: failed to get robot box.");
    return KD_ERROR;
  }

  //create RoadMap
  CkwsRoadmapShPtr kwsPlusRoadmap = CkwsRoadmap::create(robotBox) ;
  if (!kwsPlusRoadmap) {
    ODEBUG1(":walkInitMain: failed to create roadmap.");
    return KD_ERROR;
  }

  //create Diffusion Node Picker
  CkwsDiffusionNodePickerShPtr picker = CkwsPickerBasic::create ();

  //create Diffusion Shooter
  CkwsShooterConfigSpaceShPtr shooterCfgSp =  CkwsShooterConfigSpace::create();
  //CkwsShooterRoadmapBoxShPtr shooterCfgSp =  CkwsShooterRoadmapBox::create();
	
  if ( !shooterCfgSp )
  {
    ODEBUG1(":walkInitMain: failed to create the shooterCfgSp");
    return KD_ERROR;
  }

  // Create a roadmap builder strategie
  CkwsIPPRdmBuilderShPtr kwsRoadmapBuilder = 
    CkwsIPPRdmBuilder::create(kwsPlusRoadmap, picker, shooterCfgSp) ;

  //    CkwsIPPRdmBuilder::create(kwsPlusRoadmap, PENETRATION_BOX,
  //		      , picker, shooterCfgSp) ;

  if (!kwsRoadmapBuilder) {
    ODEBUG1(":walkInitMain: failed to create the roadmap.");
    return KD_ERROR;
  }

  kwsRoadmapBuilder->diffuseFromProblemStart (true);
  kwsRoadmapBuilder->diffuseFromProblemGoal (true);

  inHppPlanner->roadmapBuilderIthProblem(0, kwsRoadmapBuilder);

  // Create a Path Optimizer
  // CkwsRandomOptimizerShPtr kwsOptimizer = CkwsRandomOptimizer::create(robotBox->specificDistance());
  // kwsOptimizer->doesClear(true);  

  CkwsAdaptiveShortcutOptimizerShPtr kwsOptimizer =  CkwsAdaptiveShortcutOptimizer::create();
  kwsOptimizer->distance(robotBox->specificDistance());
  kwsOptimizer->shortcutMethod(CkwsShortcutDirect::create());

  kwsOptimizer->maxNbLoop(OPT_MAX_NB);
  ODEBUG2(" max nb loop for the optimizer "<< kwsOptimizer->maxNbLoop ());

  inHppPlanner->pathOptimizerIthProblem(0, kwsOptimizer) ;

  return KD_OK;
}


// ==========================================================================

CkppCommandLoadHRP2::CkppCommandLoadHRP2(CkppInterfaceWalk *kpp)
{
  attKpp = kpp;
}


// ==========================================================================

CkppCommandLoadHRP2::CkppCommandLoadHRP2(const CkppCommandLoadHRP2& i_command) :
  CkppCommand(i_command)
{
  attKpp = i_command.attKpp;
}


// ==========================================================================

CkppCommandLoadHRP2::~CkppCommandLoadHRP2()
{
  
}


// ==========================================================================

CkppCommandLoadHRP2ShPtr CkppCommandLoadHRP2::create(CkppInterfaceWalk *kpp)
{
  CkppCommandLoadHRP2*  ptr = new CkppCommandLoadHRP2(kpp);
  CkppCommandLoadHRP2ShPtr shPtr(ptr);
  
  if(KD_OK != ptr->init(shPtr))
    {
      shPtr.reset();
    }

  return shPtr;
}

// ==========================================================================

CkppCommandLoadHRP2ShPtr CkppCommandLoadHRP2::createCopy(const CkppCommandLoadHRP2ConstShPtr& i_command)
{
  CkppCommandLoadHRP2*  ptr = new CkppCommandLoadHRP2(*i_command);
  CkppCommandLoadHRP2ShPtr shPtr(ptr);
  
  if(KD_OK != ptr->init(shPtr))
    {
      shPtr.reset();
    }
  
  return shPtr;
}


// ==========================================================================

ktStatus CkppCommandLoadHRP2::init(const CkppCommandLoadHRP2WkPtr& i_weakPtr)
{
  ktStatus success = CkppCommand::init(i_weakPtr);
  
  if(KD_OK == success)
	{
	  m_weakPtr = i_weakPtr;
	}

  return success;
}


// ==========================================================================


CkppCommandShPtr CkppCommandLoadHRP2::clone() const
{
  return CkppCommandLoadHRP2::createCopy(m_weakPtr.lock());
}


// ==========================================================================

bool CkppCommandLoadHRP2::isUndoable() const
{
  return true;
}


// ==========================================================================

unsigned int CkppCommandLoadHRP2::countParameters() const
{
  return PARAMETER_COUNT;
}


// ==========================================================================

CkppParameterConstShPtr CkppCommandLoadHRP2::parameter(unsigned int i_rank) const
{
 
  CkppParameterShPtr result;
  CkppComponentShPtr nullComponent;

  switch(i_rank)
    {   
    default:
      KIT_ASSERT( false );
    }
  
  return result;
}


// ==========================================================================

ktStatus CkppCommandLoadHRP2::doExecute()
{
  std::string pathToHrp2(KPPINTERFACEWALK_OPENHRP_PREFIX);
  pathToHrp2 += "/Controller/IOserver/robot/HRP2JRL/model/HRP2JRLmain.wrl";

  ChppWalkPlanner *hppWalkPlanner = attKpp->hppWalkPlanner();
  if (hppWalkPlanner == NULL )
    {
      cerr << "You need to create a Planner object first." << endl;
      return KD_ERROR;
    }
  
  ChppciOpenHrpClient openHrpClient(hppWalkPlanner);
  if (openHrpClient.loadHrp2Model(0.02, pathToHrp2.c_str()) != KD_OK)
    {
      return KD_ERROR;
    }

  
#ifdef CASA_OBSTACLE
  string file_casa = "CASA/envSqrOpenHRP.wrl";
  string name_casa = "CASA";
  CkppKCDPolyhedronShPtr poly_casa;

  if (openHrpClient.loadObstacleModel(file_casa, name_casa, poly_casa) != KD_OK)
    {
      return KD_ERROR;
    }

  hppWalkPlanner->addObstacle(poly_casa);
#endif

  // Create bounding box robot and place it at beginning of problem vector.
  hppWalkPlanner->initializeProblem();
  if (setDefaultRoadmapBuilderAndOptimizer(hppWalkPlanner) != KD_OK) {
    return KD_ERROR;
  }

  ChppDeviceShPtr hrp2 = KIT_DYNAMIC_PTR_CAST(ChppDevice, hppWalkPlanner->robotIthProblem(1));
  if (hrp2) {
    ODEBUG1("HRP2 IN HALF-SITTING POSTURE");
    ODEBUG1("");
    ODEBUG1(*hrp2);
  }
  
  // Bound joints
  // Tx
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(0)->isBounded(true);
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(0)->bounds(-0.5,2.5);
  // Ty
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(1)->isBounded(true);
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(1)->bounds(-1.5,1.5);
  //Rz
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(5)->isBounded(true);
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(5)->bounds(-2*M_PI,2*M_PI);

  return KD_OK ;
 
}


// ==========================================================================
