

/*****************************************
 INCLUDES
*******************************************/
#include <iostream>

#include "kppInterface/kppCommandStartCorbaServer.h"
#include "kppInterface/kppCommandSolvePlanner.h"

#include "kppInterfaceWalk/kppInterfaceWalk.h"
#include "kppInterfaceWalk/kppCommandLoadHRP2.h"
#include "kppInterfaceWalk/kppCommandSetConfig.h"
#include "kppInterfaceWalk/kppCommandOutputMotion.h"
#include "kppInterfaceWalk/kppCommandLoadObstacle.h"


// #include "hppPolyhedron.h"
#include "hppCorbaServer/hppciOpenHrp.h"
#include "hppCore/hppProblem.h"

#include "KineoGUI/kppMainWindowUICommandFactory.h"
#include "KineoController/kppUICommandList.h"
#include "KineoController/kppUICommand.h"
#include "KineoController/kppCommandEnvironment.h"
#include "KineoController/kppCommand.h"

#include "KineoModel/kppModelTree.h"
#include "KineoModel/kppGeometryNode.h"
#include "KineoModel/kppDeviceNode.h"
#include "KineoModel/kppPathNode.h"
#include "KineoModel/kppPathComponent.h"


#include "KineoController/kppInsertComponentCommand.h"
#include "KineoController/kppInsertSolidComponentCommand.h"
#include "KineoController/kppDocument.h"
#include "KineoGUI/kppMainWindowController.h"

#include "KineoUtility/kitNotification.h"
#include "KineoUtility/kitNotificator.h"

#include "hppWalkPlanner/hppWalkPlanner.h"
#include "kppInterfaceWalk/openHrpConfig.h"

// Select level of verbosity at configuration.
#if DEBUG==3
#define ODEBUG3(x) std::cout << "CkppInterfaceWalk::" << x << std::endl
#define ODEBUG2(x) std::cout << "CkppInterfaceWalk::" << x << std::endl
#define ODEBUG1(x) std::cerr << "CkppInterfaceWalk::" << x << std::endl
#elif DEBUG==2
#define ODEBUG3(x) 
#define ODEBUG2(x) std::cout << "CkppInterfaceWalk::" << x << std::endl
#define ODEBUG1(x) std::cerr << "CkppInterfaceWalk::" << x << std::endl
#elif DEBUG==1
#define ODEBUG3(x) 
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "CkppInterfaceWalk::" << x << std::endl
#else
#define ODEBUG3(x) 
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif


using namespace std;

/*****************************************
 DEFINES
*******************************************/

/*****************************************
 METHODS
*******************************************/

// ==========================================================================

CkppInterfaceWalk::CkppInterfaceWalk(ChppWalkPlanner* inHppPlanner) : 
  CkppInterface(inHppPlanner), 
  attHppWalkPlanner(inHppPlanner)
{
}

// ==========================================================================

CkppInterfaceWalk::~CkppInterfaceWalk()
{
  delete attHppWalkPlanner; 
}

// ==========================================================================

CkppInterfaceWalkShPtr CkppInterfaceWalk::create()
{
  std::string pathToHrp2(KPPINTERFACEWALK_OPENHRP_PREFIX);
  pathToHrp2 += "/Controller/IOserver/robot/HRP2JRL/model/HRP2JRLmain.wrl";
  // Create an ChppWalkPlanner object.
  ODEBUG2(" path to HRP2: " << pathToHrp2);
  ChppWalkPlanner *planner = new ChppWalkPlanner(0.2, 0.2, 0.2, 0.04,
						 pathToHrp2.c_str());
  if (!planner) {
    cerr << "ERROR - CkppInterfaceWalk::createHppWalkPlanner: allocation of ChppWalkPlanner object failed." << endl;
    return CkppInterfaceWalkShPtr();
  }

  CkppInterfaceWalk* ptr = new CkppInterfaceWalk(planner);
  CkppInterfaceWalkShPtr shPtr(ptr);
  CkppInterfaceWalkWkPtr wkPtr(shPtr);

  if (ptr->init(wkPtr) != KD_OK) {
    shPtr.reset();
  }
  return shPtr;
}

// ==========================================================================

ktStatus CkppInterfaceWalk::init(const CkppInterfaceWalkWkPtr& inWeakPtr)
{
  if (CkppInterface::init(inWeakPtr) != KD_OK) {
    return KD_ERROR;
  }
  attWeakPtr = inWeakPtr;
  return KD_OK;
}


std::string CkppInterfaceWalk::name() const
{
  return "KPP INTERFACE WALK";
}

// ==========================================================================

void CkppInterfaceWalk::getMenuUICommandLists(const CkppMainWindowUICommandFactoryConstShPtr& i_commandFactory,
					     std::vector< CkppUICommandListShPtr > & o_menuCommandListVector)
{
  /*
    Get and store pointer to the main window controller. This is the only place where we
    have access to this object
  */
  getMainWindowController(i_commandFactory);
  // ---  LIST --- //
  CkppUICommandListShPtr hppUICommandList = CkppUICommandList::create("HPP/WALK");
 

  // --- COMMAND --- //
 
  attCommandLoadHRP2 = CkppUICommand::create(CkppCommandLoadHRP2::create(this),
						  i_commandFactory->environment(),
						  "Load HRP2",
						  "Load HRP2");

  attCommandSolvePlanner = CkppUICommand::create(CkppCommandSolvePlanner::create(static_cast<CkppInterface*>(this)),
						  i_commandFactory->environment(),
						  "Solve Problem",
						  "Solve Planner Problem");

  attCommandSetConfig = CkppUICommand::create(CkppCommandSetConfig::create(this),
						  i_commandFactory->environment(),
						  "set Init and Goal Configs",
						  "set Init and Goal Configs");

  attCommandOutputMotion = CkppUICommand::create(CkppCommandOutputMotion::create(this),
						  i_commandFactory->environment(),
						  "output motion data",
						  "output motion data");

  attStartCorbaServerCommand = CkppUICommand::create(CkppCommandStartCorbaServer::create(this),
						     i_commandFactory->environment(),
						     "Start CORBA Server",
						     "start corba server");

  attCommandLoadObstacle = CkppUICommand::create(CkppCommandLoadObstacle::create(this),
							       i_commandFactory->environment(),
							       "Load obstacle",
							       "Load obstacle");
  
  hppUICommandList->appendCommand(attCommandLoadHRP2) ; 
  hppUICommandList->appendCommand(attCommandSetConfig) ; 
  hppUICommandList->appendCommand(attCommandSolvePlanner) ; 
  hppUICommandList->appendCommand(attCommandOutputMotion) ; 
  hppUICommandList->appendCommand(attStartCorbaServerCommand) ; 
  hppUICommandList->appendCommand(attCommandLoadObstacle) ; 

  o_menuCommandListVector.push_back(hppUICommandList);
 
}

// ==========================================================================

void CkppInterfaceWalk::hppWalkPlanner(ChppWalkPlanner *planner ) 
{
  attHppWalkPlanner = planner;
  hppPlanner(planner);
}


// ------------- initializing module ------------------
// extern Function : First called by KineoWorks to Load the Module

int initializeModule(CkppModuleInterfaceShPtr& o_moduleInterface)
{
  cerr<<" initializing module ... "<<endl;
  o_moduleInterface = CkppInterfaceWalk::create();
  return 0;
}

